unit ClassInfoU;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, TypInfo, Math, StrUtils, LazUTF8;

function GetClassPropertyAsString(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer = -1): String;
function GetClassPropertyAsWideString(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer = -1): WideString;
function GetClassPropertyAsInteger(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer = -1): Integer;
function GetClassPropertyAsInt64(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer = -1): Int64;
function GetClassPropertyAsFloat(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer = -1): Double;
function GetClassPropertyAsBoolean(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer = -1): Boolean;


implementation

function GetClassPropertyValue(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer; out aTypeKind: TTypeKind): Variant;
var
  i, c: Integer;
  pi: PPropinfo;
  pl: PPropList;
begin
  aTypeKind := tkUnknown;
  Result := nil;
  c := GetPropList(aObject, pl);
  try
    for i := 0 to c - 1 do
    begin
      pi := pl^[i];
      if ((aPropertyName <> '') and (LowerCase(pi^.Name) = LowerCase(aPropertyName))) or
        ((aPropertyName = '') and (pi^.Index = aPropertyIndex)) then
      begin
        aTypeKind := pi^.PropType^.Kind;
        case pi^.PropType^.Kind of
          tkInteger, tkInt64, tkQWord, tkBool: Result := GetOrdProp(aObject, pi);
          tkFloat: Result := GetFloatProp(aObject, pi);
          tkSString, tkLString, tkAString, tkChar: Result := GetStrProp(aObject, pi);
          tkUString, tkWString, tkUChar, tkWChar: Result := GetWideStrProp(aObject, pi);
          else
            Result := GetOrdProp(aObject, pi);
        end;
        break;
      end;
    end;
  finally
    FreeMem(pl);
  end;
end;

function GetClassPropertyAsString(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer): String;
var
  tk: TTypeKind;
  v: Variant;
begin
  Result := '';
  v := GetClassPropertyValue(aObject, aPropertyName, aPropertyIndex, tk);
  if tk = tkUnknown then
    exit;
  case tk of
    tkInteger, tkInt64, tkQWord: Result := IntToStr(v);
    tkFloat: Result := FormatFloat('0.######', v);
    tkBool: Result := IfThen(v <> 0, 'true', 'false');
    tkSString, tkLString, tkAString, tkChar: Result := v;
    tkUString, tkWString, tkUChar, tkWChar: Result := UTF16ToUTF8(v);
  end;
end;

function GetClassPropertyAsWideString(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer): WideString;
var
  tk: TTypeKind;
  v: Variant;
begin
  Result := '';
  v := GetClassPropertyValue(aObject, aPropertyName, aPropertyIndex, tk);
  if tk = tkUnknown then
    exit;
  case tk of
    tkInteger, tkInt64, tkQWord: Result := IntToStr(v);
    tkFloat: Result := FormatFloat('0.######', v);
    tkBool: Result := IfThen(v <> 0, 'true', 'false');
    tkSString, tkLString, tkAString, tkChar: Result := UTF8ToUTF16(v);
    tkUString, tkWString, tkUChar, tkWChar: Result := v;
  end;
end;

function GetClassPropertyAsInteger(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer): Integer;
var
  tk: TTypeKind;
  v: Variant;
  i: Integer;
begin
  Result := 0;
  v := GetClassPropertyValue(aObject, aPropertyName, aPropertyIndex, tk);
  if tk = tkUnknown then
    exit;
  case tk of
    tkInteger, tkInt64, tkQWord: Result := v;
    tkFloat: Result := trunc(v);
    tkBool: Result := v;
    tkSString, tkLString, tkAString, tkChar:
      begin
        if TryStrToInt(v, i) then
          Result := i;
      end;
    tkUString, tkWString, tkUChar, tkWChar:
      begin
        if TryStrToInt(v, i) then
          Result := i;
      end;
  end;
end;

function GetClassPropertyAsInt64(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer): Int64;
var
  tk: TTypeKind;
  v: Variant;
  i: Int64;
begin
  Result := 0;
  v := GetClassPropertyValue(aObject, aPropertyName, aPropertyIndex, tk);
  if tk = tkUnknown then
    exit;
  case tk of
    tkInteger, tkInt64, tkQWord: Result := v;
    tkFloat: Result := trunc(v);
    tkBool: Result := v;
    tkSString, tkLString, tkAString, tkChar:
      begin
        if TryStrToInt64(v, i) then
          Result := i;
      end;
    tkUString, tkWString, tkUChar, tkWChar:
      begin
        if TryStrToInt64(v, i) then
          Result := i;
      end;
  end;
end;

function GetClassPropertyAsFloat(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer): Double;
var
  tk: TTypeKind;
  v: Variant;
  d: Double;
begin
  Result := 0;
  v := GetClassPropertyValue(aObject, aPropertyName, aPropertyIndex, tk);
  if tk = tkUnknown then
    exit;
  case tk of
    tkInteger, tkInt64, tkQWord: Result := v;
    tkFloat: Result := v;
    tkBool: Result := v;
    tkSString, tkLString, tkAString, tkChar:
      begin
        if TryStrToFloat(v, d) then
          Result := d;
      end;
    tkUString, tkWString, tkUChar, tkWChar:
      begin
        if TryStrToFloat(v, d) then
          Result := d;
      end;
  end;
end;

function GetClassPropertyAsBoolean(aObject: TObject; aPropertyName: String; aPropertyIndex: Integer): Boolean;
var
  tk: TTypeKind;
  v: Variant;
  b: Boolean;
begin
  Result := False;
  b := False;
  v := GetClassPropertyValue(aObject, aPropertyName, aPropertyIndex, tk);
  if tk = tkUnknown then
    exit;
  case tk of
    tkInteger, tkInt64, tkQWord: b := v <> 0;
    tkFloat: b := CompareValue(v, 0) <> EqualsValue;
    tkBool: b := v;
    tkSString, tkLString, tkAString, tkChar: b := (v <> '0') and (LowerCase(v) <> 'false');
    tkUString, tkWString, tkUChar, tkWChar: b := (v <> '0') and (LowerCase(v) <> 'false');
  end;
  Result := b;
end;

end.

